/**
 * \file Config.cpp
 *
 *  \date Jan 31, 2012
 *  \author Alexandre Tuleu
 */

#include "Config.h"

#include <libwebots/communication/client.hh>
#include <libwebots/plugin/modules/jointmanager/jointmanagerbuilder.hh>

namespace liboncilla {
namespace webots {
namespace config {
	PassiveDynamicsSection::PassiveDynamicsSection(const options::Section::Ptr & base)
		: options::ConfigSection(base)
		, d_hStiffness(Section())
		, d_hPreCompression(Section())
		, d_hDamping(Section())
		, d_hMaxLength(Section())
		, d_hMinLength(Section()){
	}

	PassiveDynamicsSection::~PassiveDynamicsSection(){}

	void PassiveDynamicsSection::SendJointDefinition(uint32_t legIdentifier) const {
		using namespace libwebots::communication;
		using namespace libwebots::plugin::modules::messages;

		Client & client = Client::Instance();

		client.Send(builders::JointManager::SetJointPassiveDynamic(legIdentifier,
		                                                           Stiffness(),
		                                                           Damping(),
		                                                           PreCompression()));
	}


	LegSection::LegSection(options::Section::Ptr base)
		: options::ConfigSection(base)
		, d_shDiagonal(Section())
		, d_shParallel(Section())
		, d_shFoot(Section())
		, d_hDiagonalErp(Section())
		, d_hDiagonalCfm(Section())
		, d_hMargin(Section())
		, d_hMaxSpeed(Section()){}

	LegSection::~LegSection(){}


	ContactSection::ContactSection(options::Section::Ptr parent)
		: options::ConfigSection(parent)
		, d_hBounce(Section())
		, d_hBounceVelocity(Section())
		, d_hCoulombFriction(Section())
		, d_hFds(Section())
		, d_hErp(Section())
		, d_hCfm(Section())
		, d_hDraw(Section()){}

	ContactSection::~ContactSection(){};


	MainSection::MainSection(options::Section::Ptr parent)
		: options::ConfigSection(parent)
		, d_hDebugLibwebots(Section())
		, d_hInitTime(Section())
		, d_hProcessLoopTime(Section()){
	}

	MainSection::~MainSection(){}

	Config::Config()
		: options::Config("liboncilla-webots")
		, d_shForeLeg(Section())
		, d_shHindLeg(Section())
		, d_shContact(Section())
		, d_shMain(Section()){
	}

	Config::~Config(){}

	void Config::LoadConfigFile(){
		this->LoadFile("../../config/liboncilla-webots.config");
	}

}
} /* namespace liboncilla */
} /* namespace webots */

