/**
 * \file Config.h
 *
 *  \date Jan 31, 2012
 *  \author Alexandre Tuleu
 */

#ifndef LIBONCILLA_WEBOTS_CONFIG_H_
#define LIBONCILLA_WEBOTS_CONFIG_H_


#include <biorob-cpp/options/Config.h>

/**
 *
 */

namespace liboncilla {
namespace webots {
namespace config{


	class PassiveDynamicsSection : options::ConfigSection {
	public :
		void SendJointDefinition(uint32_t leg_identifier) const;

		PassiveDynamicsSection(const options::Section::Ptr & base);
		virtual ~PassiveDynamicsSection();

		BCPP_ADD_OPTION(Stiffness,"stiffness",double,0.0,
		                "stiffness of the spring")
		BCPP_ADD_OPTION(PreCompression,"precompression",double,0.0,
		                "precompression of the spring in length/angle")
		BCPP_ADD_OPTION(Damping,"damping",double,0.0,
		                "damping of the spring")
		BCPP_ADD_OPTION(MaxLength,"max-length",double,0.0,
		                "maximal length of the spring")
		BCPP_ADD_OPTION(MinLength,"min-length",double,0.0,
		                "minimal length of the spring")

	};




	class LegSection : public options::ConfigSection{
	public :
		LegSection(options::Section::Ptr base);
		virtual ~LegSection();

		BCPP_ADD_SUBSECTION(PassiveDynamicsSection,Diagonal,"diagonal-spring",
		                    "diagonal spring properties")
		BCPP_ADD_SUBSECTION(PassiveDynamicsSection,Parallel,"parallel-spring",
		                    "parrallel spring properties")
		BCPP_ADD_SUBSECTION(PassiveDynamicsSection,Foot,"foot-spring",
		                    "foot spring properties")

		BCPP_ADD_OPTION(DiagonalErp,"erp",double,0.4,
		                "erp of the diagonal spring");
		BCPP_ADD_OPTION(DiagonalCfm,"cfm",double,2e-3,
		                "cfm of the diagonal spring")
		BCPP_ADD_OPTION(Margin,"diagonal-margin",double,0.05,
		                "margin if the diagonal spring")
		BCPP_ADD_OPTION(MaxSpeed,"max-speed",double,1.0,
		                "maximal speed reachable by the diagonal spring")

};


	class ContactSection : public options::ConfigSection {
	public :
		ContactSection(options::Section::Ptr base);
		virtual ~ContactSection();

		BCPP_ADD_OPTION(Bounce,"bounce", double,0.5,
		                "bounce webots parameter")

		BCPP_ADD_OPTION(BounceVelocity,"bounce-velocity",double,0.01,
		                "bounce-velocity webots parameter")

		BCPP_ADD_OPTION(CoulombFriction,"coulomb",double,1.0,
		                "coulomb friction coefficient")

		BCPP_ADD_OPTION(Fds,"fds",double,0.0,
		                "fds webots/ODE parameter")

		BCPP_ADD_OPTION(Erp,"erp",double,0.2,"contact erp parameter")
		BCPP_ADD_OPTION(Cfm,"cfm",double,0.2,"contact cfm parameter")

		BCPP_ADD_NON_REF_OPTION(Draw,"draw",bool,false,
		                        "wether to draw or not the contact")

	};



	class MainSection : public options::ConfigSection{
	public :
		MainSection(options::Section::Ptr base);
		virtual ~MainSection();

		BCPP_ADD_NON_REF_OPTION(DebugLibwebots,"debug-libwebots",bool,false,
		                        "add (ultragously) verbose information display about libwebots")

		BCPP_ADD_OPTION(InitTime,"init-time",double,1.5,
		                "Initialization time of the robot. When the simulation "
		                "is started, then it is runned for that amount of time, "
		                "whithout any user input.")

		BCPP_ADD_OPTION(ProcessLoopTime,"process-loop-time",double,0.006,
		                "Process loop desired time. This give an hint to the "
		                "desired simulated time that should happen between two "
		                "subsequent calls to rci::oncilla:Synchronizer::process()"
		                ". Please remind that this may not be the actual time "
		                "spent between two calls. This time is reported by "
		                "rci::oncilla:Synchronizer::lastProcessLoopTime(). "
		                "It may vary, according to the load on the system. "
		                "In simulation, it is a virtual *simulated* time, which"
		                "is not realtime. It has *nothing* to do with the actual"
		                " real time your computer spent between the two last "
		                "call to process().")

	};

	class Config : public options::Config {
	public :
		Config();
		virtual ~Config();

		void LoadConfigFile();

		BCPP_ADD_SUBSECTION(LegSection,ForeLeg,"fore-leg",
		                    "All parameters for the fore legs")
		BCPP_ADD_SUBSECTION(LegSection,HindLeg,"hind-leg",
		                    "All parameters for the hind legs")

		BCPP_ADD_SUBSECTION(ContactSection,Contact,"contact",
		                    "contact configuration")

		BCPP_ADD_SUBSECTION(MainSection,Main,"simulation",
		                    "global simulation settings")

	private :
	};

};
} /* namespace liboncilla */
} /* namespace webots */

#endif // LIBONCILLA_WEBOTS_CONFIG_H_
