/*
 * OncillaWebotsSynchronizer.cpp
 *
 *  Created on: 15 d�c. 2011
 *      Author: Alexandre Tuleu
 */
#include "Synchronizer.h"

#include <liboncilla-webots/nodes/NodeAdapter.h>
#include <webots/Supervisor.hpp>

#include <libwebots/communication/client.hh>
#include <libwebots/plugin/modules/debug/debugbuilder.hh>


namespace liboncilla {

namespace webots {


Synchronizer::Synchronizer(const WSupervisorPtr & supervisor,
                           const config::MainSection & config)
	: rci::oncilla::Synchronizer("Oncilla Webots Synchronizer")
	, d_supervisor(supervisor)
	, d_processTimestep(config.ProcessLoopTime())
	, d_webotsTimestep(std::max(1,(int)d_supervisor->getBasicTimeStep()))
	, d_initTime(config.InitTime()){
	//check first timestep validity
	int pTSinMs = 1000 * d_processTimestep;
	if( (pTSinMs % d_webotsTimestep ) != 0){
		std::ostringstream os;
		os << "simulation.process-loop-time ( here :" << d_processTimestep
		   << " s) setting is not a multiple of webots::BasicTimeStep (value : "
		   << d_supervisor->getBasicTimeStep()
		   << " ms). Please update config/liboncilla-webots.config accordingly.";
		throw std::invalid_argument(os.str());
	}

	d_nbOfWebotsStepPerProcessStep = pTSinMs / d_webotsTimestep;
	if(d_nbOfWebotsStepPerProcessStep <= 0){
		//\todo : make this exception much more clear
		throw std::invalid_argument("Uncorrect webots basic time steop setting");
	}

	using namespace libwebots::communication;
	using namespace libwebots::plugin::modules::messages::debug;
	Client & c = Client::Instance();

	c.Connect();

	if(config.DebugLibwebots()){
		Debug message;
		message.set_action(Action::Enable);
		message.set_type(Type::Info);
		c.Send(Messages::Create(debug,message));
	}


}

Synchronizer::~Synchronizer(){}

void Synchronizer::calibrateIfNeeded(){
	this->start();
	using namespace libwebots::communication;
	while(d_supervisor->getTime() < d_initTime){
		Step();
	}

}

void Synchronizer::ProcessAsyncPrimpl(){
}

void Synchronizer::WaitForProcessAsyncPrimpl(){
	Step();
}


void Synchronizer::Step(){
	using namespace libwebots::communication;

	for(unsigned int i = 0; i< d_nbOfWebotsStepPerProcessStep; ++i){

		NodeAdapter::IterateOverAllInstances1(std::mem_fun(&NodeAdapter::PreStep),0.001 * d_webotsTimestep);
		d_supervisor->step(d_webotsTimestep);
		Client::Instance().ProcessMessages();
		NodeAdapter::IterateOverAllInstances1(std::mem_fun(&NodeAdapter::PostStep),0.001 * d_webotsTimestep);
	}

}

double Synchronizer::lastProcessTimeStep() const{
	return 0.001 * d_nbOfWebotsStepPerProcessStep * d_webotsTimestep;
}


bool Synchronizer::start() {
	NodeAdapter::IterateOverAllInstances(std::mem_fun(&NodeAdapter::Activate));
	return rci::oncilla::Synchronizer::start();
}

bool Synchronizer::stop() {
	NodeAdapter::IterateOverAllInstances(std::mem_fun(&NodeAdapter::Deactivate));
	return rci::Synchronizer::stop();
}

} /* namespace liboncilla */

} /* namespace liboncilla */



