/*
 * WebotsOncillaBackend.h
 *
 *  Created on: Jan 22, 2013
 *      Author: tuleu
 */

#ifndef LIBONCILLA_WEBOTS_WEBOTSONCILLABACKEND_H_
#define LIBONCILLA_WEBOTS_WEBOTSONCILLABACKEND_H_

#include <liboncilla/utils/OncillaBackend.h>

#include "Common.h"
#include "Config.h"
#include "LegParameters.h"

class WebotsNodeWrapper;


namespace liboncilla {
namespace webots {

	class Synchronizer;

}
}


class WebotsOncillaBackend : public OncillaBackend {
public:

	WebotsOncillaBackend();
	virtual ~WebotsOncillaBackend();

	virtual boost::shared_ptr<rci::oncilla::Synchronizer> CreateSynchronizer();
	virtual boost::shared_ptr<rci::oncilla::L0> CreateL0(rci::oncilla::Leg,
	                                                     const std::string&);
	virtual boost::shared_ptr<rci::oncilla::L1L2> CreateL1(rci::oncilla::Leg,
	                                                       const std::string&);
	virtual boost::shared_ptr<rci::oncilla::L1L2> CreateL2(rci::oncilla::Leg,
	                                                       const std::string &);
	virtual boost::shared_ptr<rci::oncilla::L3> CreateL3(rci::oncilla::Leg,
	                                                     const std::string &);
	virtual boost::shared_ptr<rci::oncilla::Trunk> CreateTrunk();

	virtual boost::shared_ptr<rci::oncilla::SupervisorTrunk> CreateSupervisorTrunk();

	virtual boost::shared_ptr<rci::oncilla::SupervisorL4> CreateSupervisorL4(rci::oncilla::Leg,
	                                                                         const std::string&);
private :
	typedef boost::shared_ptr<liboncilla::webots::Synchronizer> SynchronizerPtr;
	typedef std::vector<LegParameters>                          ListOfLegParameters;
	typedef std::vector<boost::shared_ptr<rci::oncilla::L2> >   L2Vec;
	typedef std::vector<boost::shared_ptr<rci::oncilla::SupervisorL4> >   SupervisorL4Vec;
	enum JointIdentifier {
		TRUNK_0 = 1,
		L0_L1,
		L1_L2,
		L2_L3,
		L1_D1,
		D1_D2,
		D2_L3,
		L1_P1,
		P1_P2,
		P2_L3,
		L3_FOOT,
		P1_L3
	};

	const static char* LEG_PREFIXES[4];

	void Init();
	void InitIfNeeded();
	uint32_t GetJointIdentifier(rci::oncilla::Leg l,
	                            JointIdentifier j);

	void ExtractLegParametersAndInitModel(rci::oncilla::Leg l,
	                          LegParameters & params);
	void ExtractLegParameters(const WebotsNodeWrapper & l1,
	                          const WebotsNodeWrapper & l2,
	                          const WebotsNodeWrapper & l3,
	                          const WebotsNodeWrapper & p1,
	                          const WebotsNodeWrapper & foot,
	                          const std::string & d2name,
	                          LegParameters & params);

	const liboncilla::webots::config::LegSection & LegConfigSection(rci::oncilla::Leg l);


	WSupervisorPtr d_supervisor;
	//only one synchronizer per program !!!
	SynchronizerPtr d_synchronizer;

	liboncilla::webots::config::Config d_config;
	ListOfLegParameters                d_legParameters;

};

#endif /* LIBONCILLA_WEBOTS_WEBOTSONCILLABACKEND_H_ */
