/**
 * \file L1.cpp
 *
 *  \date Jan 22, 2013
 *  \author tuleu
 */

#include "L1.h"

#include <liboncilla-webots/Common.h>
#include <webots/Servo.hpp>
#include <liboncilla-webots/WebotsNodeWrapper.h>

namespace liboncilla {
namespace webots {

L1::L1(rci::oncilla::Synchronizer & s,
       const std::string & name,
       ::webots::Servo * servo,
       const WebotsNodeWrapper & wNode)
	: rci::oncilla::L1(s,name)
	, d_servo(*servo)
	, d_initialPasses(0)
	, d_minPosition(wNode.GetField<double>("minPosition"))
	, d_maxPosition(wNode.GetField<double>("maxPosition"))
	, d_maxServoForce(wNode.GetField<double>("maxForce"))
	, d_maxVelocity(wNode.GetField<double>("maxVelocity")){
	if(!servo){
		throw std::out_of_range("Could not find a webots servo node for '"
		                        + name + "'");
	}
}

L1::~L1(){
}


void L1::Activate(){
	d_servo.enablePosition(1);
}


void L1::PreStep(double ts){
	if(d_initialPasses == 0){
		d_lastServoPosition = d_servo.getPosition();
		d_rate = 0.0;
		++d_initialPasses;
	} else {
		d_rate = d_servo.getPosition() - d_lastServoPosition;
		d_rate /= ts;
		d_lastServoPosition = d_servo.getPosition();
	}

	rci::ControlModePtr c = getControlMode();
	if(c->supports(rci::jointPositionControl)){


		d_desiredPosition = clamp(getLastPositionCommand()->rad(0),
		                          d_minPosition,d_maxPosition);


		d_servo.setPosition(d_desiredPosition);
		d_servo.setVelocity(d_maxVelocity);

	}else if (c->supports(rci::jointVelocityControl)){
		//setting servo position to infinity start velocity control
		d_servo.setPosition(INFINITY);
		d_servo.setVelocity(getLastVelocityCommand()->rad_s(0));
	} else if (c->supports(rci::jointTorqueControl)) {
		d_servo.setForce(getLastTorqueCommand()->Nm(0));
	} else {
		throw std::logic_error("you should select either position, velocity or torque control.");
	}
	double force(d_maxServoForce);

	if(c->supports(rci::jointComplianceControl)) {
		double stiff = getLastImpedanceCommand()->getStiffness(0).asDouble(0);
		double damp = getLastImpedanceCommand()->getDamping(0).asDouble(0);
		double precomp = 0;//should be in Impedance;
		force = stiff * (d_desiredPosition - d_servo.getPosition() - precomp) - damp * d_rate;
	}

	//set the maximal force that we could use
	d_servo.setMotorForce(std::abs(force));
}


void L1::PostStep(double){
	_latestJointPosition->setFromRad(0,d_servo.getPosition());
	_latestJointPosition->setFromRad(1,d_servo.getPosition());
}

} /* namespace webots */
} /* namespace liboncilla */
