/**
 * \file L2.h
 *
 *  \date Jan 22, 2013
 *  \author tuleu
 */

#ifndef LIBONCILLA_WEBOTS_L2_H_
#define LIBONCILLA_WEBOTS_L2_H_

#include <liboncilla/nodes/L1L2.h>
#include "NodeAdapter.h"
#include <liboncilla-webots/Common.h>

namespace webots {
	class Servo;
}

namespace liboncilla {
namespace webots {

namespace config {
	class LegSection;
}

class L2 : public rci::oncilla::L2,
           public NodeAdapter {
public:
	L2(rci::oncilla::Synchronizer & s,
	   const std::string & name,
       uint32_t identifier,
       ::webots::Servo * servoInline,
       ::webots::Servo * servoDiagonal,
       const config::LegSection & config);
	virtual ~L2();

	virtual void Activate();
	virtual void Deactivate();

	virtual void PreStep(double);
	virtual void PostStep(double);

	//Controlled interface stubbed
	virtual void setControlMode(rci::ControlModePtr velocity){
		NOT_IMPLEMENTED_STUB(L2,setControlMode);
	}


	//VelocityControlled interface stubbed
	virtual bool setJointVelocity(rci::JointVelocitiesPtr velocity){
		NOT_IMPLEMENTED_STUB(L2,setJointVelocity);
	}

	virtual bool setJointTorque(rci::JointTorquesPtr position){
		NOT_IMPLEMENTED_STUB(L2,setJointTorque);
	}

	virtual bool setJointImpedance(rci::JointImpedancePtr position){
		NOT_IMPLEMENTED_STUB(L2,setJointImpedance);
	}
private :
	void Init(const config::LegSection & config);

	::webots::Servo & d_servoInline;
	::webots::Servo & d_servoDiagonal;
	const uint32_t d_joint;
	unsigned int d_passes;
	double d_lastCommand;
	const double d_minPosition;
	const double d_maxPosition;
	const double d_maxSpeed;
};

} /* namespace webots */
} /* namespace liboncilla */
#endif /* LIBONCILLA_WEBOTS_L2_H_ */
