/**
 * \file NodeAdapter.h
 *
 *  \date Jan 22, 2013
 *  \author tuleu
 */

#ifndef LIBONCILLA_WEBOTS_NODEADAPTER_H_
#define LIBONCILLA_WEBOTS_NODEADAPTER_H_

#include <set>
#include <algorithm>
#include <functional>

namespace liboncilla {
namespace webots {


/**
 * generic hook interface for all nodes
 */
class NodeAdapter {
public:
	NodeAdapter();
	virtual ~NodeAdapter();

	static void IterateOverAllInstances(std::mem_fun_t<void,NodeAdapter> f);

	template <typename T>
	static void IterateOverAllInstances1(std::mem_fun1_t<void,NodeAdapter,T> f,double value);

	virtual void PreStep(double ts);

	virtual void PostStep(double ts);

	virtual void Activate();

	virtual void Deactivate();

private :
	typedef std::set<NodeAdapter*> ListOfInstances;

	static ListOfInstances s_instances;
};
template <typename T>
inline void NodeAdapter::IterateOverAllInstances1(std::mem_fun1_t<void,NodeAdapter,T> f,double value){
	for(ListOfInstances::const_iterator i = s_instances.begin();
	    i != s_instances.end();
	    ++i){
		f(*i,value);
	}
}

} /* namespace webots */
} /* namespace liboncilla */
#endif /* LIBONCILLA_WEBOTS_NODEADAPTER_H_ */
