/**
 * \file SupervisorL4.cpp
 *
 *  \date Jan 22, 2013
 *  \author tuleu
 */

#include "SupervisorL4.h"
#include <liboncilla-webots/Common.h>
#include <libwebots/communication/client.hh>
#include <libwebots/communication/messages.hh>


namespace liboncilla {
namespace webots {

SupervisorL4::ContactManager * SupervisorL4::s_contactManager(NULL);
const char * SupervisorL4::ContactManager::GROUND_NAME= "GROUND";



SupervisorL4::SupervisorL4(const std::string & name,
                           ::webots::Supervisor & supervisor,
	                        const liboncilla::webots::config::ContactSection & config,
                           const std::string & footNodeName)
	: rci::oncilla::SupervisorL4(name)
	, d_node(supervisor.getFromDef(footNodeName)){
	if(d_node == NULL){
		throw std::runtime_error("Could not find suitable node " + footNodeName
		                         + " for SupervisorL4" + name);
	}

	if(s_contactManager == NULL ){
		s_contactManager = new ContactManager();
	}
	s_contactManager->RegisterNewContact(footNodeName,config,this);

}

SupervisorL4::~SupervisorL4(){
	s_contactManager->Unregister(this);
}

void SupervisorL4::PreStep(double){
	d_force.setZero();
}


void SupervisorL4::PostStep(double){
	const double * P(d_node->getPosition());
	const double * R(d_node->getOrientation());

	if(P == NULL || R == NULL){
		throw std::runtime_error("COuld not read position for " + this->nodename);
	}

	updatePose(rci::Translation::fromMeters(P[0],P[1],P[2]),
	           rci::Orientation::fromRotationMatrix(R[0],R[1],R[2],
	                                                R[3],R[4],R[5],
	                                                R[6],R[7],R[8]));

	updateForces(rci::ForcesPtr(new rci::Forces(d_force[0],d_force[1],d_force[2])));
}


SupervisorL4::ContactManager::ContactManager(){
	libwebots::communication::Client::Instance().AddHandler(*this,
	                                                        libwebots::plugin::modules::messages::contact::contactmonitor,
	                                                        &ContactManager::Handler);
}

SupervisorL4::ContactManager::~ContactManager(){
}

void SupervisorL4::ContactManager::Handler(const MonitorMessage & m){
	NodeByName::const_iterator n = d_nodes.find(m.name());
	if( n == d_nodes.end()){
		n = d_nodes.find(m.other());
	}

	if( n == d_nodes.end()){
		return;
	}

	unsigned int nbContact = m.force_size() / 3;
	for(unsigned int i = 0; i < nbContact; ++i){

		n->second->d_force += Eigen::Vector3d(m.force(3 * i),
		                                      m.force(3 * i + 1),
		                                      m.force(3 * i + 2));

	}
}


void SupervisorL4::ContactManager::RegisterNewContact(const std::string & name,
                                                      const liboncilla::webots::config::ContactSection & config,
                                                      SupervisorL4 * node){
	if(d_nodes.insert(std::make_pair(name,node)).second == false){
		return;
	}
	if(d_names.insert(std::make_pair(node,name)).second == false){
		d_nodes.erase(name);
		return;
	}

	using namespace libwebots::communication;
	using namespace libwebots::plugin::modules::messages::contact;
	Contact m;
	m.set_body1(name);
	m.set_body2(GROUND_NAME);
	m.set_coulombfriction(config.CoulombFriction());
	m.set_draw(config.Draw());
	m.set_monitor(true);
	m.set_softerp(config.Erp());
	m.set_softcfm(config.Cfm());
	//m.set_computedirection(true);


	Client::Instance().Send(Messages::Create(contact,m));


}

void SupervisorL4::ContactManager::Unregister(SupervisorL4 * node){
	NameByNode::iterator n = d_names.find(node);
	if(n == d_names.end()){
		return;
	}
	d_nodes.erase(n->second);
	d_names.erase(n);
}

} /* namespace webots */
} /* namespace liboncilla */
