/**
 * \file Trunk.cpp
 *
 *  \date Jan 22, 2013
 *  \author tuleu
 */

#include "Trunk.h"

#include <liboncilla-webots/Common.h>

namespace liboncilla {
namespace webots {

Trunk::Trunk(::webots::Node * node)
	: d_node(node)
	, d_initialized(0){
	if(node == NULL){
		throw std::runtime_error("Coquld not find suitable node for teh trunk.");
	}
}

Trunk::~Trunk(){
}

void Trunk::PostStep(double ts){
	const double * posVal(d_node->getPosition());
	if(posVal == NULL){
		throw std::runtime_error("Could not find Trunk current position.");
	}
	Eigen::Vector3d newPos(posVal);
	if(d_initialized == 2){
		Eigen::Vector3d newVel = (newPos - d_position) / ts;
		Eigen::Vector3d accel = (newVel - d_velocity) / ts;
		d_velocity = newVel;
		updateTranslationalAcceleration(rci::TranslationalAcceleration::fromM_s2(accel[0],accel[1],accel[2]));
	} else if(d_initialized == 1){
		d_velocity = (newPos - d_position) /ts;
		updateTranslationalAcceleration(rci::TranslationalAcceleration::fromM_s2(0,0,0));
		++ d_initialized;
	} else if (d_initialized == 0){
		updateTranslationalAcceleration(rci::TranslationalAcceleration::fromM_s2(0,0,0));
		++d_initialized;
	}
	d_position = newPos;

	const double * ori(d_node->getOrientation());
	if(ori == NULL){
		throw std::runtime_error("Could not find Trunk Orientation.");
	}
	updateOrientation(rci::Orientation::fromRotationMatrix(ori[0],ori[1],ori[2],
	                                                       ori[3],ori[4],ori[5],
	                                                       ori[6],ori[7],ori[8]));

}



} /* namespace webots */
} /* namespace liboncilla */
